"""Module containing functionality for determining the default `Target` from the current system."""

import ast
import re
from enum import StrEnum
from pathlib import Path


class Target(StrEnum):
    """Target for generated spec files."""

    FEDORA = "fedora"
    OPENSUSE = "opensuse"
    EPEL8 = "epel8"
    MAGEIA = "mageia"
    PLAIN = "plain"
    """Target for build environments with minimal RPM macros but no dependency generators."""


TARGET_NAMES = [target.value for target in Target]


def read_os_release() -> dict[str, str]:
    """Return parsed contents of the os-release file.

    Returns:
        Mapping from keys to values.

    """
    try:
        with Path("/etc/os-release").open() as file:
            contents = file.read()
    except FileNotFoundError:
        with Path("/usr/lib/os-release").open() as file:
            contents = file.read()

    data = {}

    for line in contents.splitlines():
        stripped = line.rstrip()
        if not stripped or stripped.startswith("#"):
            continue
        if m := re.match(r"([A-Z][A-Z_0-9]+)=(.*)", stripped):
            name, val = m.groups()
            if val and val[0] in "\"'":
                val = ast.literal_eval(val)
            data[name] = val

    return data


def get_default_target() -> Target:
    """Return default `Target` based on the current system.

    Returns:
        Default target for the current system.

    """
    os_release = read_os_release()
    os_id = os_release.get("ID")

    # ID_LIKE is a space-separated list of identifiers like ID
    os_like = os_release.get("ID_LIKE", "").split()

    # Order matters here!
    if "mageia" in (os_id, *os_like):
        return Target.MAGEIA
    if "fedora" in (os_id, *os_like):
        return Target.FEDORA
    if "suse" in os_like:
        return Target.OPENSUSE
    return Target.PLAIN
